from crosswalk_client.validators.client import validate_required_threshold


class SetThreshold(object):
    @validate_required_threshold
    def set_threshold(self, threshold):
        self.threshold = threshold



from crosswalk_client.exceptions import MalformedBlockAttributes


def validate_required_block_attrs_arg(function):
    def wrapper(*args, **kwargs):
        block_attrs = args[1]
        if not isinstance(block_attrs, dict):
            raise MalformedBlockAttributes(
                "Block attributes must be a dictionary"
            )
        for key in block_attrs:
            if isinstance(block_attrs[key], dict):
                raise MalformedBlockAttributes(
                    "Nested attributes are not allowed in block attributes"
                )
        return function(*args, **kwargs)

    return wrapper



from uuid import UUID

from crosswalk_client.exceptions import MalformedUUID


def validate_required_uuid_arg(function):
    def wrapper(*args, **kwargs):
        uuid = args[1]
        if not isinstance(uuid, UUID):
            raise MalformedUUID("Invalid UUID")
        return function(*args, **kwargs)

    return wrapper



from urllib.parse import urljoin

import requests

from crosswalk_client.exceptions import ConfigError


class ClientCheck(object):
    """Check if client was configured correctly and is authorized."""

    def client_check(self):
        response = requests.get(
            urljoin(self.service_address, "client-check/"),
            headers=self.headers,
        )
        if response.status_code != requests.codes.ok:
            raise ConfigError(
                "Client configuration error. Check that the token and service "
                "address are correctly configured."
            )
        return response.status_code == requests.codes.ok



from slugify import slugify

from crosswalk_client.exceptions import MalformedDomain
from crosswalk_client.objects.domain import DomainObject


def validate_parent_domain_kwarg(function):
    """
    Validates a domain is passed. Converts DomainObject instances to slug.
    """

    def wrapper(*args, **kwargs):
        parent = kwargs.get("parent", None)
        if parent is None:
            return function(*args, **kwargs)
        if isinstance(parent, DomainObject):
            kwargs["parent"] = parent.slug
            return function(*args, **kwargs)
        if not isinstance(parent, str):
            raise MalformedDomain(
                "Parent domain must be a string or domain instance"
            )
        if slugify(parent) != parent:
            raise MalformedDomain("Parent domain string should be a slug")
        return function(*args, **kwargs)

    return wrapper



from urllib.parse import urljoin

import requests

from crosswalk_client.exceptions import BadResponse
from crosswalk_client.objects.entity import EntityObject
from crosswalk_client.validators.entity import (
    validate_block_attrs_kwarg,
    validate_domain_kwarg,
)


class GetEntities(object):
    @validate_block_attrs_kwarg
    @validate_domain_kwarg
    def get_entities(self, block_attrs={}, domain=None):
        if domain is None:
            domain = self.domain
        response = requests.get(
            urljoin(
                self.service_address, "domains/{}/entities/".format(domain)
            ),
            headers=self.headers,
            params=block_attrs,
        )
        if response.status_code != requests.codes.ok:
            raise BadResponse(
                "The service responded with a {}: {}".format(
                    response.status_code, response.content
                )
            )
        entities = response.json()
        return [
            EntityObject({"entity": entity}, client=self)
            for entity in entities
        ]



from crosswalk_client.exceptions import MalformedThreshold, MissingThreshold


def validate_required_threshold(function):
    """
    Validates a threshold.
    """

    def wrapper(*args, **kwargs):
        list_args = list(args)
        try:
            threshold = list_args[1]
        except IndexError:
            raise MissingThreshold("Must pass a threshold")
        if not isinstance(threshold, int):
            raise MalformedThreshold("Threshold must be an integer")
        if threshold < 0 or threshold > 100:
            raise MalformedThreshold("Threshold must be between 0 and 100")
        return function(*args, **kwargs)

    return wrapper



from crosswalk_client.exceptions import MalformedQuery


def validate_required_query_arg(function):
    def wrapper(*args, **kwargs):
        query = args[1]
        if not isinstance(query, dict):
            raise MalformedQuery("Query should be a dictionary")
        if len(query.keys()) > 1:
            raise MalformedQuery(
                "Query should only include one key value pair"
            )
        if not isinstance(list(query.values())[0], str) or not isinstance(
            list(query.keys())[0], str
        ):
            raise MalformedQuery("Query key and value should be strings")
        return function(*args, **kwargs)

    return wrapper



from urllib.parse import urljoin

import requests

from crosswalk_client.encoder import encode
from crosswalk_client.exceptions import (
    BadResponse,
    UnspecificQueryError,
    EntityNotFoundError,
)
from crosswalk_client.objects.entity import EntityObject
from crosswalk_client.validators.entity import (
    validate_block_attrs_kwarg,
    validate_domain_kwarg,
    validate_required_query_arg,
)


class Match(object):
    @validate_required_query_arg
    @validate_block_attrs_kwarg
    @validate_domain_kwarg
    def match(
        self,
        query,
        block_attrs={},
        create_attrs={},
        domain=None,
        return_canonical=True,
    ):
        if domain is None:
            domain = self.domain
        query_field = list(query.keys())[0]
        data = {
            "query_field": query_field,
            "query_value": query[query_field],
            "block_attrs": block_attrs,
            "return_canonical": return_canonical,
        }
        response = requests.post(
            urljoin(
                self.service_address,
                "domains/{}/entities/match/".format(domain),
            ),
            headers=self.headers,
            data=encode(data),
        )
        if response.status_code == 403:
            raise UnspecificQueryError(
                "Query in request matched more than one entity."
            )
        if response.status_code == 404:
            raise EntityNotFoundError("No entity found for the given query")
        if response.status_code != requests.codes.ok:
            raise BadResponse(
                "The service responded with a {}: {}".format(
                    response.status_code, response.content
                )
            )
        return EntityObject(response.json(), client=self)



from crosswalk_client.exceptions import MalformedScorer, MissingScorer

VALID_SCORERS = [
    "fuzzywuzzy.default_process",
    "fuzzywuzzy.partial_ratio_process",
    "fuzzywuzzy.token_sort_ratio_process",
    "fuzzywuzzy.token_set_ratio_process",
]


def validate_required_scorer(function):
    """
    Validates a threshold.
    """

    def wrapper(*args, **kwargs):
        list_args = list(args)
        try:
            scorer = list_args[1]
        except IndexError:
            raise MissingScorer("Must pass a scorer")
        if not isinstance(scorer, str):
            raise MalformedScorer("Scorer must be a string")
        if scorer not in VALID_SCORERS:
            raise MalformedScorer("Invalid scorer")
        return function(*args, **kwargs)

    return wrapper



from crosswalk_client import Client


def test_setup(token, service):
    client = Client(token, service)
    client.create_domain("presidents")
    client.create_domain("politicians")


def test_domain_update(token, service):
    client = Client(token, service)
    domain = client.get_domain("presidents")
    domain.update({"name": "Presidents"})
    assert domain.name == "Presidents"


def test_domain_set_parent(token, service):
    client = Client(token, service)
    domain = client.get_domain("politicians")
    domain.set_parent(client.get_domain("presidents"))
    assert domain.parent == "presidents"


def test_domain_remove_parent(token, service):
    client = Client(token, service)
    domain = client.get_domain("politicians")
    domain.remove_parent()
    assert domain.parent is None


def test_domain_delete(token, service):
    client = Client(token, service)
    domain = client.get_domain("politicians")
    domain.delete()
    assert domain.deleted is True

    domain = client.get_domain("presidents")
    domain.delete()
    assert domain.deleted is True



from uuid import UUID

from crosswalk_client.exceptions import MalformedUUID


def validate_target_uuid_arg(function):
    """
    Target UUIDs are used to set foreign keys and should be either a valid UUID
    or None to unset the foreign key.
    """

    def wrapper(*args, **kwargs):
        uuid = args[2]
        if uuid is not None and not isinstance(uuid, UUID):
            raise MalformedUUID("Invalid UUID for target")
        return function(*args, **kwargs)

    return wrapper



from .methods import ClientMethods


class Client(ClientMethods):
    def __init__(
        self,
        token,
        service_address,
        domain=None,
        threshold=80,
        scorer="fuzzywuzzy.default_process",
    ):
        self.token = token
        self.service_address = service_address
        self.headers = {
            "Authorization": "TOKEN {}".format(self.token),
            "Content-Type": "application/json",
        }
        self.domain = domain
        self.threshold = threshold
        self.scorer = scorer
        self.client_check()



import os

from setuptools import find_packages, setup

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="django-crosswalk-client",
    version="0.0.11",
    packages=find_packages(exclude=("example",)),
    include_package_data=True,
    license="MIT",
    url="https://github.com/The-Politico/django-crosswalk-client",
    description="Client companion to django-crosswalk.",
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    install_requires=["requests", "python-slugify", "stringcase"],
)
